type token =
  | AMPERSAND
  | AT_SIGN
  | CARET
  | CNF
  | COLON
  | COMMA
  | EQUALS
  | EXCLAMATION
  | FOF
  | GETS
  | GREATER
  | HOF
  | IF
  | IFF
  | IMPLIES
  | INCLUDE_TOK
  | INPUT_CLAUSE
  | INPUT_FORMULA
  | LAMBDA
  | LBRKT
  | LPAREN
  | MAP_TO
  | MMINUS
  | NAMPERSAND
  | NEQUALS
  | NIFF
  | NVLINE
  | PERIOD
  | PPLUS
  | QUESTION
  | RBRKT
  | RPAREN
  | TILDE
  | TOK_FALSE
  | TOK_I
  | TOK_O
  | TOK_INT
  | TOK_REAL
  | TOK_TRUE
  | TOK_TYPE
  | VLINE
  | EOF
  | DTHF
  | DFOF
  | DCNF
  | DFOT
  | Unsigned_integer of (string)
  | Signed_integer of (string)
  | Decimal_part of (string)
  | Real of (string)
  | Atomic_system_word of (string)
  | Distinct_object of (string)
  | Single_quoted of (string)
  | Upper_word of (string)
  | Lower_word of (string)
  | Unrecognized

open Parsing;;
# 3 "parser-hotptp/htparser.mly"

open Hol_type
open Term
open Signature
open Lexing


let print_error startpos endpos msg =
	if startpos.pos_lnum = endpos.pos_lnum then
		Printf.printf "line %d, characters %d-%d: %s\n"
		startpos.pos_lnum (startpos.pos_cnum - startpos.pos_bol)
		(endpos.pos_cnum - endpos.pos_bol)
		msg
	else
		Printf.printf "line %d, character %d - line %d, character %d: %s\n"
		startpos.pos_lnum (startpos.pos_cnum - startpos.pos_bol)
		endpos.pos_lnum (endpos.pos_cnum - endpos.pos_bol)
		msg

let terms = ref []
let sigma = ref (new_signature ())
let bound_typevars = ref (Hashtbl.create 17)
let typevar_counter = ref 0
let termroles = ref (Hashtbl.create 3)
let add_fun = ref (fun _ _ -> ())

let reset () = let t= !terms in
               let s= !sigma in
               let r= !termroles in
               terms := [];
               sigma := new_signature ();
               termroles := (Hashtbl.create 3);
               (t,s,r)

let reset_bound_typevars () =
  bound_typevars := Hashtbl.create 17;
  typevar_counter := 0

let add_typevar s =
  Hashtbl.add !bound_typevars s (!typevar_counter);
  typevar_counter := !typevar_counter + 1

let add_term t =
  terms := t :: !terms;
  add_fun := fun n r -> Hashtbl.add !termroles r (n,t)


let add_def (s,t) =
  try (
    let ty = Term.type_of (type_of_symbol !sigma) t in
    add_defined_symbol ~ty:(Some ty) !sigma s t
  ) with Failure msg ->
    begin
      print_string ("\nin definition "^s^":\n"^msg^"\n\n");
      add_defined_symbol !sigma s t
    end;
  add_fun := fun n r -> Hashtbl.add !termroles r (s,t)

let add_const_strict (c,ty) =
  add_uninterpreted_symbol !sigma c ty;
  add_fun := fun n r -> Hashtbl.add !termroles r (c, Symbol c)

let add_const (c,ty) =
  try (
    let ty2 = type_of_symbol !sigma c in
    (* print_string ("Symbol "^c^": ."^(Hol_type.to_string ty2)^"., ."^(Hol_type.to_string ty)^".\n"); *)
    if not (ty2 = ty)
    then failwith ("add_const: type mismatch for symbol "^c^" and types "^
                   (Hol_type.to_string ty2)^" and "^(Hol_type.to_string ty2))
  ) with Failure _ -> add_const_strict (c,ty)

# 133 "parser-hotptp/htparser.ml"
let yytransl_const = [|
  257 (* AMPERSAND *);
  258 (* AT_SIGN *);
  259 (* CARET *);
  260 (* CNF *);
  261 (* COLON *);
  262 (* COMMA *);
  263 (* EQUALS *);
  264 (* EXCLAMATION *);
  265 (* FOF *);
  266 (* GETS *);
  267 (* GREATER *);
  268 (* HOF *);
  269 (* IF *);
  270 (* IFF *);
  271 (* IMPLIES *);
  272 (* INCLUDE_TOK *);
  273 (* INPUT_CLAUSE *);
  274 (* INPUT_FORMULA *);
  275 (* LAMBDA *);
  276 (* LBRKT *);
  277 (* LPAREN *);
  278 (* MAP_TO *);
  279 (* MMINUS *);
  280 (* NAMPERSAND *);
  281 (* NEQUALS *);
  282 (* NIFF *);
  283 (* NVLINE *);
  284 (* PERIOD *);
  285 (* PPLUS *);
  286 (* QUESTION *);
  287 (* RBRKT *);
  288 (* RPAREN *);
  289 (* TILDE *);
  290 (* TOK_FALSE *);
  291 (* TOK_I *);
  292 (* TOK_O *);
  293 (* TOK_INT *);
  294 (* TOK_REAL *);
  295 (* TOK_TRUE *);
  296 (* TOK_TYPE *);
  297 (* VLINE *);
    0 (* EOF *);
  298 (* DTHF *);
  299 (* DFOF *);
  300 (* DCNF *);
  301 (* DFOT *);
  311 (* Unrecognized *);
    0|]

let yytransl_block = [|
  302 (* Unsigned_integer *);
  303 (* Signed_integer *);
  304 (* Decimal_part *);
  305 (* Real *);
  306 (* Atomic_system_word *);
  307 (* Distinct_object *);
  308 (* Single_quoted *);
  309 (* Upper_word *);
  310 (* Lower_word *);
    0|]

let yylhs = "\255\255\
\001\000\001\000\001\000\001\000\001\000\004\000\005\000\007\000\
\009\000\009\000\012\000\014\000\014\000\014\000\015\000\015\000\
\015\000\015\000\015\000\015\000\020\000\020\000\020\000\020\000\
\016\000\016\000\017\000\017\000\013\000\013\000\025\000\003\000\
\003\000\003\000\028\000\028\000\028\000\027\000\027\000\030\000\
\030\000\026\000\026\000\031\000\031\000\031\000\008\000\010\000\
\023\000\023\000\023\000\033\000\033\000\022\000\022\000\036\000\
\036\000\037\000\038\000\038\000\040\000\040\000\041\000\041\000\
\035\000\035\000\035\000\035\000\042\000\045\000\045\000\046\000\
\043\000\048\000\034\000\034\000\034\000\049\000\049\000\049\000\
\050\000\050\000\052\000\052\000\024\000\024\000\024\000\053\000\
\053\000\047\000\051\000\021\000\021\000\054\000\054\000\056\000\
\058\000\058\000\058\000\058\000\058\000\058\000\058\000\058\000\
\057\000\057\000\057\000\059\000\059\000\060\000\060\000\061\000\
\061\000\055\000\055\000\055\000\055\000\055\000\062\000\063\000\
\063\000\064\000\065\000\065\000\065\000\065\000\065\000\029\000\
\029\000\029\000\029\000\029\000\029\000\029\000\029\000\029\000\
\029\000\029\000\070\000\069\000\069\000\069\000\069\000\069\000\
\069\000\069\000\069\000\069\000\069\000\039\000\039\000\039\000\
\039\000\039\000\039\000\073\000\044\000\044\000\072\000\071\000\
\071\000\067\000\067\000\066\000\066\000\074\000\068\000\068\000\
\075\000\032\000\032\000\032\000\032\000\032\000\076\000\076\000\
\076\000\076\000\076\000\076\000\077\000\078\000\079\000\079\000\
\006\000\006\000\011\000\011\000\018\000\019\000\019\000\019\000\
\002\000\000\000"

let yylen = "\002\000\
\001\000\002\000\002\000\002\000\001\000\010\000\010\000\001\000\
\003\000\001\000\001\000\001\000\003\000\001\000\001\000\004\000\
\001\000\001\000\001\000\001\000\004\000\004\000\004\000\004\000\
\002\000\003\000\001\000\003\000\002\000\001\000\001\000\010\000\
\010\000\010\000\001\000\001\000\001\000\003\000\005\000\001\000\
\001\000\005\000\003\000\001\000\001\000\001\000\001\000\001\000\
\001\000\003\000\003\000\001\000\002\000\001\000\001\000\001\000\
\001\000\003\000\001\000\001\000\003\000\003\000\003\000\001\000\
\001\000\001\000\001\000\003\000\006\000\001\000\003\000\001\000\
\002\000\001\000\001\000\003\000\004\000\001\000\001\000\001\000\
\001\000\001\000\001\000\003\000\001\000\001\000\004\000\001\000\
\001\000\001\000\001\000\001\000\001\000\001\000\001\000\003\000\
\001\000\001\000\001\000\001\000\001\000\001\000\001\000\001\000\
\001\000\001\000\001\000\003\000\003\000\003\000\003\000\003\000\
\003\000\001\000\001\000\001\000\001\000\003\000\006\000\006\000\
\006\000\004\000\001\000\001\000\001\000\001\000\001\000\001\000\
\001\000\003\000\001\000\001\000\001\000\002\000\003\000\001\000\
\001\000\001\000\001\000\001\000\001\000\001\000\001\000\001\000\
\001\000\001\000\001\000\001\000\001\000\001\000\001\000\001\000\
\001\000\001\000\001\000\006\000\001\000\001\000\003\000\006\000\
\006\000\001\000\001\000\001\000\003\000\003\000\001\000\003\000\
\003\000\001\000\001\000\001\000\003\000\003\000\001\000\001\000\
\001\000\001\000\001\000\001\000\001\000\001\000\001\000\001\000\
\001\000\001\000\001\000\001\000\001\000\001\000\001\000\001\000\
\000\000\002\000"

let yydefred = "\000\000\
\000\000\000\000\005\000\000\000\001\000\000\000\000\000\000\000\
\002\000\003\000\004\000\000\000\000\000\000\000\186\000\188\000\
\187\000\000\000\185\000\000\000\000\000\000\000\000\000\000\000\
\000\000\008\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\079\000\078\000\190\000\191\000\192\000\189\000\000\000\
\000\000\090\000\089\000\048\000\000\000\000\000\052\000\086\000\
\075\000\000\000\085\000\157\000\000\000\158\000\074\000\000\000\
\047\000\067\000\000\000\055\000\056\000\057\000\000\000\000\000\
\065\000\066\000\000\000\000\000\146\000\163\000\148\000\143\000\
\140\000\142\000\162\000\000\000\147\000\149\000\141\000\145\000\
\000\000\045\000\044\000\144\000\046\000\123\000\125\000\000\000\
\000\000\000\000\000\000\000\000\000\000\092\000\000\000\094\000\
\095\000\000\000\000\000\000\000\114\000\115\000\116\000\117\000\
\000\000\126\000\127\000\000\000\053\000\000\000\010\000\000\000\
\081\000\082\000\000\000\000\000\000\000\000\000\000\000\000\000\
\153\000\150\000\152\000\155\000\151\000\154\000\000\000\000\000\
\000\000\000\000\073\000\000\000\000\000\041\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\103\000\100\000\097\000\099\000\102\000\104\000\098\000\
\101\000\000\000\000\000\000\000\000\000\000\000\000\000\051\000\
\000\000\000\000\000\000\000\000\000\000\019\000\000\000\000\000\
\011\000\000\000\014\000\017\000\018\000\020\000\000\000\000\000\
\076\000\000\000\050\000\000\000\000\000\068\000\000\000\061\000\
\058\000\062\000\063\000\000\000\000\000\072\000\000\000\000\000\
\118\000\000\000\000\000\000\000\124\000\000\000\000\000\000\000\
\038\000\000\000\176\000\175\000\177\000\178\000\179\000\181\000\
\180\000\000\000\170\000\171\000\172\000\000\000\000\000\000\000\
\110\000\112\000\108\000\096\000\109\000\111\000\113\000\000\000\
\000\000\000\000\000\000\025\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\030\000\009\000\000\000\007\000\
\000\000\000\000\000\000\006\000\000\000\000\000\000\000\000\000\
\000\000\122\000\034\000\032\000\033\000\000\000\184\000\183\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\026\000\000\000\000\000\000\000\000\000\000\000\031\000\
\029\000\013\000\000\000\084\000\000\000\071\000\000\000\039\000\
\042\000\174\000\000\000\000\000\000\000\165\000\000\000\000\000\
\000\000\000\000\168\000\028\000\021\000\022\000\023\000\024\000\
\016\000\087\000\069\000\119\000\120\000\121\000\000\000\169\000"

let yydgoto = "\002\000\
\004\000\111\000\009\000\010\000\011\000\018\000\025\000\056\000\
\112\000\040\000\085\000\168\000\238\000\229\000\170\000\171\000\
\230\000\042\000\043\000\174\000\133\000\057\000\044\000\045\000\
\017\001\089\000\090\000\000\000\000\000\091\000\197\000\028\001\
\046\000\058\000\059\000\060\000\061\000\062\000\127\000\063\000\
\064\000\065\000\066\000\093\000\188\000\189\000\048\000\068\000\
\049\000\115\000\050\000\181\000\051\000\094\000\095\000\096\000\
\097\000\155\000\098\000\099\000\100\000\101\000\102\000\103\000\
\104\000\215\000\105\000\226\000\106\000\107\000\000\000\000\000\
\000\000\216\000\227\000\211\000\212\000\213\000\001\001"

let yysindex = "\020\000\
\083\255\000\000\000\000\185\255\000\000\006\255\077\255\090\255\
\000\000\000\000\000\000\133\255\133\255\133\255\000\000\000\000\
\000\000\085\255\000\000\123\255\126\255\110\255\110\255\110\255\
\165\255\000\000\172\255\174\255\194\255\175\000\040\000\194\255\
\164\000\000\000\000\000\000\000\000\000\000\000\000\000\176\255\
\000\000\000\000\000\000\000\000\070\255\098\255\000\000\000\000\
\000\000\131\255\000\000\000\000\175\000\000\000\000\000\176\255\
\000\000\000\000\144\000\000\000\000\000\000\000\199\255\163\255\
\000\000\000\000\191\255\175\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\094\000\000\000\000\000\000\000\000\000\
\192\255\000\000\000\000\000\000\000\000\000\000\000\000\176\255\
\176\255\176\255\211\255\230\255\216\255\000\000\075\001\000\000\
\000\000\196\255\238\255\242\255\000\000\000\000\000\000\000\000\
\229\255\000\000\000\000\220\255\000\000\089\001\000\000\222\255\
\000\000\000\000\204\255\194\255\204\255\240\255\241\255\175\000\
\000\000\000\000\000\000\000\000\000\000\000\000\175\000\175\000\
\175\000\209\255\000\000\148\000\248\255\000\000\002\000\004\000\
\148\000\249\255\251\255\253\255\148\000\239\255\209\255\148\000\
\148\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\148\000\148\000\148\000\148\000\148\000\209\255\000\000\
\076\001\012\000\014\000\017\000\018\000\000\000\019\000\008\000\
\000\000\037\000\000\000\000\000\000\000\000\000\016\000\000\000\
\000\000\024\000\000\000\043\000\025\000\000\000\022\000\000\000\
\000\000\000\000\000\000\247\255\045\000\000\000\148\000\041\000\
\000\000\148\000\239\255\026\000\000\000\028\000\034\000\035\000\
\000\000\239\255\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\124\255\000\000\000\000\000\000\047\000\038\000\062\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\066\000\
\044\000\046\000\070\000\000\000\072\000\054\000\148\000\175\000\
\194\255\204\255\089\001\052\000\000\000\000\000\089\001\000\000\
\204\255\204\255\000\000\000\000\077\000\209\255\239\255\056\000\
\127\255\000\000\000\000\000\000\000\000\152\255\000\000\000\000\
\239\255\239\255\085\000\209\255\116\001\093\000\095\000\209\255\
\089\001\000\000\067\000\074\000\078\000\080\000\091\000\000\000\
\000\000\000\000\097\000\000\000\175\000\000\000\127\255\000\000\
\000\000\000\000\124\255\124\255\148\000\000\000\000\000\148\000\
\148\000\120\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\086\000\000\000"

let yyrindex = "\000\000\
\001\000\000\000\000\000\130\001\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\248\255\
\246\000\000\000\000\000\000\000\000\000\044\255\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\248\255\
\000\000\000\000\038\255\000\000\000\000\000\000\054\255\067\255\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\027\255\000\000\000\000\000\000\000\000\000\000\000\000\248\255\
\116\255\248\255\000\000\224\000\000\000\000\000\072\255\000\000\
\000\000\081\255\088\255\102\255\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\011\001\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\171\255\248\255\
\000\000\049\255\000\000\000\000\000\000\000\000\000\000\090\000\
\000\000\000\000\000\000\100\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\103\000\000\000\000\000\032\001\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\039\255\000\000\000\000\000\000\000\000\000\000\105\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\106\000\000\000\065\255\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\054\001\000\000\000\000\000\000\000\000\000\000\
\128\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\057\255\084\255\000\000\000\000\119\255\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000"

let yygindex = "\000\000\
\000\000\007\000\000\000\000\000\000\000\154\000\193\000\000\000\
\013\000\000\000\246\255\000\000\000\000\162\255\000\000\159\000\
\052\255\255\255\051\000\000\000\243\255\210\255\224\255\187\255\
\000\000\066\001\000\000\000\000\000\000\068\001\236\255\146\255\
\000\000\233\255\193\255\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\250\255\160\000\000\000\143\255\000\000\
\000\000\000\000\213\255\234\255\000\000\000\000\137\255\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\112\255\000\000\145\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000"

let yytablesize = 682
let yytable = "\108\000\
\193\000\019\000\019\000\019\000\131\000\047\000\118\000\005\000\
\047\000\109\000\092\000\026\000\026\000\026\000\225\000\169\000\
\190\000\088\000\041\000\041\000\001\000\041\000\041\000\067\000\
\217\000\218\000\012\000\139\000\139\000\086\000\015\001\210\000\
\139\000\139\000\219\000\220\000\221\000\222\000\223\000\139\000\
\139\000\139\000\041\000\054\000\043\000\177\000\067\000\180\000\
\043\000\049\000\139\000\139\000\139\000\139\000\012\000\136\000\
\184\000\041\000\139\000\059\000\036\001\067\000\173\000\185\000\
\186\000\187\000\173\000\139\000\119\000\054\000\043\000\178\000\
\060\000\178\000\086\000\049\000\113\000\093\000\064\000\012\000\
\012\000\087\000\003\000\179\000\249\000\059\000\105\000\173\000\
\173\000\166\000\022\000\254\000\047\000\106\000\114\000\027\000\
\027\000\013\000\060\000\167\000\138\000\139\000\140\000\093\000\
\176\000\041\000\176\000\107\000\172\000\041\000\014\000\192\000\
\105\000\067\000\166\000\030\001\041\000\041\000\041\000\106\000\
\067\000\067\000\067\000\196\000\169\000\041\000\087\000\201\000\
\023\000\179\000\086\000\024\000\190\000\107\000\255\000\086\000\
\023\001\255\000\116\000\086\000\179\000\214\000\086\000\086\000\
\018\001\000\001\027\001\193\000\000\001\169\000\167\000\117\000\
\086\000\086\000\086\000\086\000\086\000\224\000\025\001\172\000\
\173\000\016\000\255\000\017\000\014\001\044\001\020\000\021\000\
\045\001\046\001\029\000\180\000\180\000\000\001\237\000\015\000\
\015\000\030\000\015\000\031\000\248\000\110\000\087\000\026\001\
\016\000\012\001\017\000\087\000\006\000\086\000\178\000\087\000\
\086\000\007\000\087\000\087\000\008\000\178\000\178\000\128\000\
\013\001\015\000\015\000\129\000\087\000\087\000\087\000\087\000\
\087\000\047\000\130\000\173\000\137\000\043\001\032\000\027\000\
\028\000\011\001\019\001\020\001\141\000\041\000\041\000\176\000\
\167\000\067\000\033\000\034\000\167\000\086\000\176\000\176\000\
\035\000\172\000\142\000\143\000\156\000\172\000\157\000\036\000\
\037\000\087\000\038\000\158\000\087\000\016\000\039\000\017\000\
\159\000\036\000\037\000\160\000\038\000\175\000\167\000\016\000\
\039\000\017\000\214\000\202\000\193\000\039\000\034\001\172\000\
\195\000\193\000\041\000\194\000\193\000\236\000\067\000\182\000\
\183\000\203\000\204\000\205\000\206\000\245\000\207\000\193\000\
\198\000\087\000\199\000\086\000\200\000\173\000\086\000\086\000\
\231\000\173\000\232\000\208\000\209\000\233\000\234\000\235\000\
\069\000\239\000\070\000\240\000\241\000\247\000\071\000\052\000\
\242\000\244\000\246\000\002\001\072\000\073\000\074\000\251\000\
\243\000\250\000\075\000\173\000\076\000\252\000\253\000\077\000\
\078\000\079\000\080\000\004\001\003\001\054\000\005\001\161\000\
\081\000\082\000\006\001\008\001\007\001\009\001\083\000\087\000\
\084\000\021\001\087\000\087\000\010\001\036\000\037\000\024\001\
\038\000\029\001\088\000\016\000\039\000\017\000\069\000\088\000\
\070\000\032\001\037\001\033\001\071\000\052\000\088\000\088\000\
\088\000\038\001\072\000\073\000\074\000\039\001\091\000\040\001\
\075\000\088\000\132\000\088\000\088\000\077\000\078\000\079\000\
\080\000\088\000\041\001\054\000\047\001\048\001\081\000\082\000\
\042\001\194\000\088\000\083\000\083\000\070\000\084\000\164\000\
\167\000\043\000\016\001\036\000\037\000\134\000\038\000\135\000\
\120\000\016\000\039\000\017\000\069\000\022\001\070\000\000\000\
\035\001\000\000\071\000\052\000\121\000\122\000\123\000\000\000\
\072\000\073\000\074\000\000\000\000\000\000\000\075\000\124\000\
\191\000\125\000\126\000\077\000\078\000\079\000\080\000\000\000\
\000\000\054\000\000\000\000\000\081\000\082\000\052\000\000\000\
\000\000\000\000\083\000\000\000\084\000\000\000\000\000\000\000\
\000\000\036\000\037\000\053\000\038\000\034\000\000\000\016\000\
\039\000\017\000\035\000\000\000\054\000\000\000\000\000\055\000\
\034\000\036\000\037\000\000\000\038\000\035\000\000\000\016\000\
\039\000\017\000\000\000\000\000\036\000\037\000\000\000\038\000\
\124\000\124\000\016\000\039\000\017\000\124\000\124\000\000\000\
\000\000\040\000\000\000\000\000\124\000\124\000\124\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\080\000\124\000\
\124\000\124\000\124\000\080\000\088\000\000\000\000\000\124\000\
\000\000\000\000\080\000\080\000\080\000\000\000\000\000\000\000\
\124\000\000\000\091\000\124\000\124\000\080\000\088\000\080\000\
\080\000\124\000\000\000\000\000\040\000\080\000\000\000\124\000\
\124\000\124\000\000\000\000\000\000\000\000\000\080\000\000\000\
\124\000\124\000\124\000\124\000\124\000\124\000\124\000\000\000\
\000\000\000\000\124\000\000\000\124\000\124\000\124\000\000\000\
\000\000\000\000\000\000\124\000\000\000\000\000\077\000\124\000\
\124\000\124\000\124\000\077\000\087\000\000\000\000\000\124\000\
\000\000\000\000\077\000\077\000\077\000\000\000\000\000\000\000\
\124\000\000\000\000\000\144\000\145\000\077\000\087\000\077\000\
\077\000\146\000\000\000\000\000\000\000\077\000\000\000\147\000\
\148\000\149\000\000\000\000\000\000\000\000\000\077\000\161\000\
\000\000\000\000\150\000\151\000\152\000\153\000\000\000\000\000\
\000\000\000\000\228\000\000\000\161\000\000\000\000\000\000\000\
\000\000\000\000\000\000\154\000\000\000\162\000\163\000\164\000\
\165\000\036\000\037\000\000\000\038\000\000\000\166\000\016\000\
\039\000\017\000\162\000\163\000\164\000\165\000\036\000\037\000\
\202\000\038\000\000\000\166\000\016\000\039\000\017\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\203\000\204\000\
\205\000\206\000\000\000\031\001\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\208\000\209\000"

let yycheck = "\032\000\
\000\000\012\000\013\000\014\000\068\000\029\000\053\000\001\000\
\032\000\033\000\031\000\022\000\023\000\024\000\159\000\110\000\
\130\000\031\000\029\000\030\000\001\000\032\000\033\000\030\000\
\144\000\145\000\021\001\001\001\002\001\031\000\235\000\142\000\
\006\001\007\001\154\000\155\000\156\000\157\000\158\000\013\001\
\014\001\015\001\053\000\006\001\006\001\115\000\053\000\117\000\
\010\001\006\001\024\001\025\001\026\001\027\001\006\001\076\000\
\120\000\068\000\032\001\006\001\009\001\068\000\006\001\127\000\
\128\000\129\000\010\001\041\001\056\000\032\001\032\001\115\000\
\006\001\117\000\076\000\032\001\007\001\006\001\041\001\031\001\
\032\001\031\000\000\001\116\000\195\000\032\001\006\001\031\001\
\032\001\006\001\006\001\202\000\116\000\006\001\025\001\031\001\
\032\001\021\001\032\001\110\000\088\000\089\000\090\000\032\001\
\115\000\116\000\117\000\006\001\110\000\120\000\021\001\132\000\
\032\001\120\000\031\001\004\001\127\000\128\000\129\000\032\001\
\127\000\128\000\129\000\137\000\006\001\010\001\076\000\141\000\
\006\001\011\001\132\000\006\001\246\000\032\001\011\001\137\000\
\247\000\011\001\041\001\141\000\022\001\143\000\144\000\145\000\
\239\000\022\001\001\001\032\001\022\001\031\001\161\000\021\001\
\154\000\155\000\156\000\157\000\158\000\159\000\032\001\161\000\
\110\000\052\001\011\001\054\001\234\000\029\001\013\000\014\000\
\032\001\033\001\006\001\241\000\242\000\022\001\168\000\005\001\
\006\001\006\001\046\001\006\001\194\000\006\001\132\000\032\001\
\052\001\232\000\054\001\137\000\004\001\191\000\234\000\141\000\
\194\000\009\001\144\000\145\000\012\001\241\000\242\000\001\001\
\233\000\031\001\032\001\041\001\154\000\155\000\156\000\157\000\
\158\000\233\000\020\001\161\000\021\001\021\001\021\001\023\000\
\024\000\231\000\241\000\242\000\010\001\232\000\233\000\234\000\
\235\000\232\000\033\001\034\001\239\000\231\000\241\000\242\000\
\039\001\235\000\005\001\020\001\041\001\239\000\001\001\046\001\
\047\001\191\000\049\001\002\001\194\000\052\001\053\001\054\001\
\020\001\046\001\047\001\032\001\049\001\032\001\009\001\052\001\
\053\001\054\001\004\001\021\001\004\001\053\001\008\001\009\001\
\005\001\009\001\021\001\010\001\012\001\006\001\021\001\032\001\
\032\001\035\001\036\001\037\001\038\001\031\001\040\001\032\001\
\032\001\231\000\032\001\029\001\032\001\235\000\032\001\033\001\
\021\001\239\000\021\001\053\001\054\001\021\001\021\001\021\001\
\001\001\005\001\003\001\028\001\021\001\005\001\007\001\008\001\
\006\001\028\001\006\001\005\001\013\001\014\001\015\001\028\001\
\032\001\032\001\019\001\009\001\021\001\028\001\028\001\024\001\
\025\001\026\001\027\001\006\001\031\001\030\001\005\001\020\001\
\033\001\034\001\031\001\006\001\031\001\006\001\039\001\029\001\
\041\001\005\001\032\001\033\001\031\001\046\001\047\001\032\001\
\049\001\005\001\001\001\052\001\053\001\054\001\001\001\006\001\
\003\001\005\001\032\001\005\001\007\001\008\001\013\001\014\001\
\015\001\032\001\013\001\014\001\015\001\032\001\021\001\032\001\
\019\001\024\001\021\001\026\001\027\001\024\001\025\001\026\001\
\027\001\032\001\032\001\030\001\005\001\040\001\033\001\034\001\
\032\001\000\000\041\001\032\001\039\001\031\001\041\001\031\001\
\031\001\010\001\236\000\046\001\047\001\076\000\049\001\076\000\
\001\001\052\001\053\001\054\001\001\001\246\000\003\001\255\255\
\008\001\255\255\007\001\008\001\013\001\014\001\015\001\255\255\
\013\001\014\001\015\001\255\255\255\255\255\255\019\001\024\001\
\021\001\026\001\027\001\024\001\025\001\026\001\027\001\255\255\
\255\255\030\001\255\255\255\255\033\001\034\001\008\001\255\255\
\255\255\255\255\039\001\255\255\041\001\255\255\255\255\255\255\
\255\255\046\001\047\001\021\001\049\001\034\001\255\255\052\001\
\053\001\054\001\039\001\255\255\030\001\255\255\255\255\033\001\
\034\001\046\001\047\001\255\255\049\001\039\001\255\255\052\001\
\053\001\054\001\255\255\255\255\046\001\047\001\255\255\049\001\
\001\001\002\001\052\001\053\001\054\001\006\001\007\001\255\255\
\255\255\010\001\255\255\255\255\013\001\014\001\015\001\255\255\
\255\255\255\255\255\255\255\255\255\255\255\255\001\001\024\001\
\025\001\026\001\027\001\006\001\007\001\255\255\255\255\032\001\
\255\255\255\255\013\001\014\001\015\001\255\255\255\255\255\255\
\041\001\255\255\021\001\001\001\002\001\024\001\025\001\026\001\
\027\001\007\001\255\255\255\255\010\001\032\001\255\255\013\001\
\014\001\015\001\255\255\255\255\255\255\255\255\041\001\255\255\
\001\001\002\001\024\001\025\001\026\001\027\001\007\001\255\255\
\255\255\255\255\032\001\255\255\013\001\014\001\015\001\255\255\
\255\255\255\255\255\255\041\001\255\255\255\255\001\001\024\001\
\025\001\026\001\027\001\006\001\007\001\255\255\255\255\032\001\
\255\255\255\255\013\001\014\001\015\001\255\255\255\255\255\255\
\041\001\255\255\255\255\001\001\002\001\024\001\025\001\026\001\
\027\001\007\001\255\255\255\255\255\255\032\001\255\255\013\001\
\014\001\015\001\255\255\255\255\255\255\255\255\041\001\020\001\
\255\255\255\255\024\001\025\001\026\001\027\001\255\255\255\255\
\255\255\255\255\031\001\255\255\020\001\255\255\255\255\255\255\
\255\255\255\255\255\255\041\001\255\255\042\001\043\001\044\001\
\045\001\046\001\047\001\255\255\049\001\255\255\051\001\052\001\
\053\001\054\001\042\001\043\001\044\001\045\001\046\001\047\001\
\021\001\049\001\255\255\051\001\052\001\053\001\054\001\255\255\
\255\255\255\255\255\255\255\255\255\255\255\255\035\001\036\001\
\037\001\038\001\255\255\040\001\255\255\255\255\255\255\255\255\
\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
\053\001\054\001"

let yynames_const = "\
  AMPERSAND\000\
  AT_SIGN\000\
  CARET\000\
  CNF\000\
  COLON\000\
  COMMA\000\
  EQUALS\000\
  EXCLAMATION\000\
  FOF\000\
  GETS\000\
  GREATER\000\
  HOF\000\
  IF\000\
  IFF\000\
  IMPLIES\000\
  INCLUDE_TOK\000\
  INPUT_CLAUSE\000\
  INPUT_FORMULA\000\
  LAMBDA\000\
  LBRKT\000\
  LPAREN\000\
  MAP_TO\000\
  MMINUS\000\
  NAMPERSAND\000\
  NEQUALS\000\
  NIFF\000\
  NVLINE\000\
  PERIOD\000\
  PPLUS\000\
  QUESTION\000\
  RBRKT\000\
  RPAREN\000\
  TILDE\000\
  TOK_FALSE\000\
  TOK_I\000\
  TOK_O\000\
  TOK_INT\000\
  TOK_REAL\000\
  TOK_TRUE\000\
  TOK_TYPE\000\
  VLINE\000\
  EOF\000\
  DTHF\000\
  DFOF\000\
  DCNF\000\
  DFOT\000\
  Unrecognized\000\
  "

let yynames_block = "\
  Unsigned_integer\000\
  Signed_integer\000\
  Decimal_part\000\
  Real\000\
  Atomic_system_word\000\
  Distinct_object\000\
  Single_quoted\000\
  Upper_word\000\
  Lower_word\000\
  "

let yyact = [|
  (fun _ -> failwith "parser")
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'null) in
    Obj.repr(
# 158 "parser-hotptp/htparser.mly"
         ( reset () )
# 640 "parser-hotptp/htparser.ml"
               : Term.term list * Signature.signature * (string , string * Term.term) Hashtbl.t))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 1 : Term.term list * Signature.signature * (string , string * Term.term) Hashtbl.t) in
    let _2 = (Parsing.peek_val __caml_parser_env 0 : 'hof_annotated) in
    Obj.repr(
# 159 "parser-hotptp/htparser.mly"
                        ( (!terms,!sigma,!termroles) )
# 648 "parser-hotptp/htparser.ml"
               : Term.term list * Signature.signature * (string , string * Term.term) Hashtbl.t))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 1 : Term.term list * Signature.signature * (string , string * Term.term) Hashtbl.t) in
    let _2 = (Parsing.peek_val __caml_parser_env 0 : 'fof_annotated) in
    Obj.repr(
# 160 "parser-hotptp/htparser.mly"
                        ( (!terms,!sigma,!termroles) )
# 656 "parser-hotptp/htparser.ml"
               : Term.term list * Signature.signature * (string , string * Term.term) Hashtbl.t))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 1 : Term.term list * Signature.signature * (string , string * Term.term) Hashtbl.t) in
    let _2 = (Parsing.peek_val __caml_parser_env 0 : 'cnf_annotated) in
    Obj.repr(
# 161 "parser-hotptp/htparser.mly"
                        ( (!terms,!sigma,!termroles) )
# 664 "parser-hotptp/htparser.ml"
               : Term.term list * Signature.signature * (string , string * Term.term) Hashtbl.t))
; (fun __caml_parser_env ->
    Obj.repr(
# 162 "parser-hotptp/htparser.mly"
          ( let startpos = Parsing.rhs_start_pos 1 in
						let endpos = Parsing.rhs_end_pos 1 in
						print_error startpos endpos "syntax error";
                                                failwith("Syntax Error")
				    (* ;(!terms,!sigma,!termroles) *))
# 674 "parser-hotptp/htparser.ml"
               : Term.term list * Signature.signature * (string , string * Term.term) Hashtbl.t))
; (fun __caml_parser_env ->
    let _3 = (Parsing.peek_val __caml_parser_env 7 : 'name) in
    let _5 = (Parsing.peek_val __caml_parser_env 5 : 'formula_role) in
    let _7 = (Parsing.peek_val __caml_parser_env 3 : 'fof_top) in
    let _8 = (Parsing.peek_val __caml_parser_env 2 : 'annotations) in
    Obj.repr(
# 171 "parser-hotptp/htparser.mly"
                                                                               (Main.set_input_logic "FOF"; !add_fun _3 _5)
# 684 "parser-hotptp/htparser.ml"
               : 'fof_annotated))
; (fun __caml_parser_env ->
    let _3 = (Parsing.peek_val __caml_parser_env 7 : 'name) in
    let _5 = (Parsing.peek_val __caml_parser_env 5 : 'formula_role) in
    let _7 = (Parsing.peek_val __caml_parser_env 3 : 'cnf_top) in
    let _8 = (Parsing.peek_val __caml_parser_env 2 : 'annotations) in
    Obj.repr(
# 175 "parser-hotptp/htparser.mly"
                                                                               (Main.set_input_logic "CNF"; !add_fun _3 _5)
# 694 "parser-hotptp/htparser.ml"
               : 'cnf_annotated))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'atomic_word) in
    Obj.repr(
# 179 "parser-hotptp/htparser.mly"
              ( _1 )
# 701 "parser-hotptp/htparser.ml"
               : 'formula_role))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 1 : 'source) in
    let _3 = (Parsing.peek_val __caml_parser_env 0 : 'optional_info) in
    Obj.repr(
# 183 "parser-hotptp/htparser.mly"
                               ()
# 709 "parser-hotptp/htparser.ml"
               : 'annotations))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'null) in
    Obj.repr(
# 184 "parser-hotptp/htparser.mly"
         ()
# 716 "parser-hotptp/htparser.ml"
               : 'annotations))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'general_term) in
    Obj.repr(
# 188 "parser-hotptp/htparser.mly"
                 ()
# 723 "parser-hotptp/htparser.ml"
               : 'source))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'general_data) in
    Obj.repr(
# 192 "parser-hotptp/htparser.mly"
                 ()
# 730 "parser-hotptp/htparser.ml"
               : 'general_term))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 2 : 'general_data) in
    let _3 = (Parsing.peek_val __caml_parser_env 0 : 'general_term) in
    Obj.repr(
# 193 "parser-hotptp/htparser.mly"
                                    ()
# 738 "parser-hotptp/htparser.ml"
               : 'general_term))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'general_list) in
    Obj.repr(
# 194 "parser-hotptp/htparser.mly"
                 ()
# 745 "parser-hotptp/htparser.ml"
               : 'general_term))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'atomic_word) in
    Obj.repr(
# 198 "parser-hotptp/htparser.mly"
                ()
# 752 "parser-hotptp/htparser.ml"
               : 'general_data))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 3 : 'atomic_word) in
    let _3 = (Parsing.peek_val __caml_parser_env 1 : 'general_terms) in
    Obj.repr(
# 199 "parser-hotptp/htparser.mly"
                                            ()
# 760 "parser-hotptp/htparser.ml"
               : 'general_data))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'variable) in
    Obj.repr(
# 200 "parser-hotptp/htparser.mly"
             ()
# 767 "parser-hotptp/htparser.ml"
               : 'general_data))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'number) in
    Obj.repr(
# 201 "parser-hotptp/htparser.mly"
           ()
# 774 "parser-hotptp/htparser.ml"
               : 'general_data))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : string) in
    Obj.repr(
# 202 "parser-hotptp/htparser.mly"
                    ()
# 781 "parser-hotptp/htparser.ml"
               : 'general_data))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'formula_data) in
    Obj.repr(
# 203 "parser-hotptp/htparser.mly"
                 ()
# 788 "parser-hotptp/htparser.ml"
               : 'general_data))
; (fun __caml_parser_env ->
    let _3 = (Parsing.peek_val __caml_parser_env 1 : 'thf_logic_formula) in
    Obj.repr(
# 207 "parser-hotptp/htparser.mly"
                                         ()
# 795 "parser-hotptp/htparser.ml"
               : 'formula_data))
; (fun __caml_parser_env ->
    let _3 = (Parsing.peek_val __caml_parser_env 1 : 'fof_formula) in
    Obj.repr(
# 208 "parser-hotptp/htparser.mly"
                                   ()
# 802 "parser-hotptp/htparser.ml"
               : 'formula_data))
; (fun __caml_parser_env ->
    let _3 = (Parsing.peek_val __caml_parser_env 1 : 'cnf_formula) in
    Obj.repr(
# 209 "parser-hotptp/htparser.mly"
                                   ()
# 809 "parser-hotptp/htparser.ml"
               : 'formula_data))
; (fun __caml_parser_env ->
    let _3 = (Parsing.peek_val __caml_parser_env 1 : 'fof_term) in
    Obj.repr(
# 210 "parser-hotptp/htparser.mly"
                                ()
# 816 "parser-hotptp/htparser.ml"
               : 'formula_data))
; (fun __caml_parser_env ->
    Obj.repr(
# 214 "parser-hotptp/htparser.mly"
                ()
# 822 "parser-hotptp/htparser.ml"
               : 'general_list))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 1 : 'general_terms) in
    Obj.repr(
# 215 "parser-hotptp/htparser.mly"
                              ()
# 829 "parser-hotptp/htparser.ml"
               : 'general_list))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'general_term) in
    Obj.repr(
# 219 "parser-hotptp/htparser.mly"
                 ()
# 836 "parser-hotptp/htparser.ml"
               : 'general_terms))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 2 : 'general_term) in
    let _3 = (Parsing.peek_val __caml_parser_env 0 : 'general_terms) in
    Obj.repr(
# 220 "parser-hotptp/htparser.mly"
                                     ()
# 844 "parser-hotptp/htparser.ml"
               : 'general_terms))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 0 : 'useful_info) in
    Obj.repr(
# 224 "parser-hotptp/htparser.mly"
                      ()
# 851 "parser-hotptp/htparser.ml"
               : 'optional_info))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'null) in
    Obj.repr(
# 225 "parser-hotptp/htparser.mly"
         ()
# 858 "parser-hotptp/htparser.ml"
               : 'optional_info))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'general_list) in
    Obj.repr(
# 229 "parser-hotptp/htparser.mly"
                 ()
# 865 "parser-hotptp/htparser.ml"
               : 'useful_info))
; (fun __caml_parser_env ->
    let _3 = (Parsing.peek_val __caml_parser_env 7 : 'name) in
    let _5 = (Parsing.peek_val __caml_parser_env 5 : 'formula_role) in
    let _7 = (Parsing.peek_val __caml_parser_env 3 : 'typed_constant) in
    let _8 = (Parsing.peek_val __caml_parser_env 2 : 'annotations) in
    Obj.repr(
# 236 "parser-hotptp/htparser.mly"
                                                                                      (Main.set_input_logic "THF"; add_const_strict _7; !add_fun _3 _5)
# 875 "parser-hotptp/htparser.ml"
               : 'hof_annotated))
; (fun __caml_parser_env ->
    let _3 = (Parsing.peek_val __caml_parser_env 7 : 'name) in
    let _5 = (Parsing.peek_val __caml_parser_env 5 : 'formula_role) in
    let _7 = (Parsing.peek_val __caml_parser_env 3 : 'definition) in
    let _8 = (Parsing.peek_val __caml_parser_env 2 : 'annotations) in
    Obj.repr(
# 237 "parser-hotptp/htparser.mly"
                                                                                  (Main.set_input_logic "THF"; add_def _7; reset_bound_typevars(); !add_fun _3 _5)
# 885 "parser-hotptp/htparser.ml"
               : 'hof_annotated))
; (fun __caml_parser_env ->
    let _3 = (Parsing.peek_val __caml_parser_env 7 : 'name) in
    let _5 = (Parsing.peek_val __caml_parser_env 5 : 'formula_role) in
    let _7 = (Parsing.peek_val __caml_parser_env 3 : 'thf_logic_formula) in
    let _8 = (Parsing.peek_val __caml_parser_env 2 : 'annotations) in
    Obj.repr(
# 238 "parser-hotptp/htparser.mly"
                                                                                         (Main.set_input_logic "THF";
            match (_5,_7) with
              ("definition",(Appl (Appl (Symbol "=", Symbol definiendum), definiens))) -> (add_def (definiendum,definiens);
                                                                                           reset_bound_typevars();
                                                                                           !add_fun _3 _5)
            | _ -> (add_term _7; reset_bound_typevars();!add_fun _3 _5))
# 900 "parser-hotptp/htparser.ml"
               : 'hof_annotated))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'term) in
    Obj.repr(
# 249 "parser-hotptp/htparser.mly"
         ( add_term _1; reset_bound_typevars() )
# 907 "parser-hotptp/htparser.ml"
               : 'hof_formula))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'definition) in
    Obj.repr(
# 250 "parser-hotptp/htparser.mly"
               ( add_def _1; reset_bound_typevars() )
# 914 "parser-hotptp/htparser.ml"
               : 'hof_formula))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'typed_constant) in
    Obj.repr(
# 251 "parser-hotptp/htparser.mly"
                   ( add_const_strict _1 )
# 921 "parser-hotptp/htparser.ml"
               : 'hof_formula))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 2 : 'defined_element) in
    let _3 = (Parsing.peek_val __caml_parser_env 0 : 'thf_logic_formula) in
    Obj.repr(
# 258 "parser-hotptp/htparser.mly"
                                           ( (_1,_3) )
# 929 "parser-hotptp/htparser.ml"
               : 'definition))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 3 : 'defined_element) in
    let _4 = (Parsing.peek_val __caml_parser_env 1 : 'thf_logic_formula) in
    Obj.repr(
# 259 "parser-hotptp/htparser.mly"
                                                        ( (_2,_4) )
# 937 "parser-hotptp/htparser.ml"
               : 'definition))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'constant) in
    Obj.repr(
# 264 "parser-hotptp/htparser.mly"
             ( _1 )
# 944 "parser-hotptp/htparser.ml"
               : 'defined_element))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'typed_constant) in
    Obj.repr(
# 265 "parser-hotptp/htparser.mly"
                   ( fst _1 )
# 951 "parser-hotptp/htparser.ml"
               : 'defined_element))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 3 : 'constant) in
    let _4 = (Parsing.peek_val __caml_parser_env 1 : 'type_expr) in
    Obj.repr(
# 269 "parser-hotptp/htparser.mly"
                                           ( (_2,_4) )
# 959 "parser-hotptp/htparser.ml"
               : 'typed_constant))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 2 : 'constant) in
    let _3 = (Parsing.peek_val __caml_parser_env 0 : 'type_expr) in
    Obj.repr(
# 270 "parser-hotptp/htparser.mly"
                             ( (_1,_3) )
# 967 "parser-hotptp/htparser.ml"
               : 'typed_constant))
; (fun __caml_parser_env ->
    Obj.repr(
# 274 "parser-hotptp/htparser.mly"
             ( Signature.ctrue )
# 973 "parser-hotptp/htparser.ml"
               : 'constant))
; (fun __caml_parser_env ->
    Obj.repr(
# 275 "parser-hotptp/htparser.mly"
              ( Signature.cfalse )
# 979 "parser-hotptp/htparser.ml"
               : 'constant))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'atomic_word) in
    Obj.repr(
# 276 "parser-hotptp/htparser.mly"
                ( _1 )
# 986 "parser-hotptp/htparser.ml"
               : 'constant))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'fof_formula) in
    Obj.repr(
# 281 "parser-hotptp/htparser.mly"
                ((* print_string ("Read: "^(Term.to_hotptp $1)^"\n\n"); *)
      add_term (multi_quantified (Symbol Signature.forall) (List.map (fun x -> (x,bt_i)) (free_vars _1)) _1);
      reset_bound_typevars() )
# 995 "parser-hotptp/htparser.ml"
               : 'fof_top))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'cnf_formula) in
    Obj.repr(
# 287 "parser-hotptp/htparser.mly"
                (
      add_term (multi_quantified (Symbol Signature.forall) (List.map (fun x -> (x,bt_i)) (free_vars _1)) _1);
      reset_bound_typevars() )
# 1004 "parser-hotptp/htparser.ml"
               : 'cnf_top))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'cnf_literal) in
    Obj.repr(
# 293 "parser-hotptp/htparser.mly"
                ( _1 )
# 1011 "parser-hotptp/htparser.ml"
               : 'cnf_formula))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 2 : 'cnf_literal) in
    let _3 = (Parsing.peek_val __caml_parser_env 0 : 'cnf_formula) in
    Obj.repr(
# 294 "parser-hotptp/htparser.mly"
                                  ( Appl(Appl(Symbol(Signature.disjunction),_1),_3) )
# 1019 "parser-hotptp/htparser.ml"
               : 'cnf_formula))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 1 : 'cnf_formula) in
    Obj.repr(
# 295 "parser-hotptp/htparser.mly"
                              ( _2 )
# 1026 "parser-hotptp/htparser.ml"
               : 'cnf_formula))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'fof_atomic_formula) in
    Obj.repr(
# 299 "parser-hotptp/htparser.mly"
                       ( _1 )
# 1033 "parser-hotptp/htparser.ml"
               : 'cnf_literal))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 0 : 'fof_atomic_formula) in
    Obj.repr(
# 300 "parser-hotptp/htparser.mly"
                             ( Appl(Symbol(Signature.neg),_2) )
# 1040 "parser-hotptp/htparser.ml"
               : 'cnf_literal))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'fof_unitary_formula) in
    Obj.repr(
# 304 "parser-hotptp/htparser.mly"
                        ( _1 )
# 1047 "parser-hotptp/htparser.ml"
               : 'fof_formula))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'fof_binary_formula) in
    Obj.repr(
# 305 "parser-hotptp/htparser.mly"
                       ( _1 )
# 1054 "parser-hotptp/htparser.ml"
               : 'fof_formula))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'fof_nonassoc_binary) in
    Obj.repr(
# 309 "parser-hotptp/htparser.mly"
                        ( _1 )
# 1061 "parser-hotptp/htparser.ml"
               : 'fof_binary_formula))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'fof_assoc_binary) in
    Obj.repr(
# 310 "parser-hotptp/htparser.mly"
                     ( _1 )
# 1068 "parser-hotptp/htparser.ml"
               : 'fof_binary_formula))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 2 : 'fof_unitary_formula) in
    let _2 = (Parsing.peek_val __caml_parser_env 1 : 'fof_binary_connective) in
    let _3 = (Parsing.peek_val __caml_parser_env 0 : 'fof_unitary_formula) in
    Obj.repr(
# 314 "parser-hotptp/htparser.mly"
                                                                  ( Appl(Appl(_2,_1),_3) )
# 1077 "parser-hotptp/htparser.ml"
               : 'fof_nonassoc_binary))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'fof_and_formula) in
    Obj.repr(
# 318 "parser-hotptp/htparser.mly"
                    ( _1 )
# 1084 "parser-hotptp/htparser.ml"
               : 'fof_assoc_binary))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'fof_or_formula) in
    Obj.repr(
# 319 "parser-hotptp/htparser.mly"
                   ( _1 )
# 1091 "parser-hotptp/htparser.ml"
               : 'fof_assoc_binary))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 2 : 'fof_unitary_formula) in
    let _3 = (Parsing.peek_val __caml_parser_env 0 : 'fof_unitary_formula) in
    Obj.repr(
# 323 "parser-hotptp/htparser.mly"
                                                     ( Appl(Appl(Symbol(Signature.conjunction),_1),_3) )
# 1099 "parser-hotptp/htparser.ml"
               : 'fof_and_formula))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 2 : 'fof_and_formula) in
    let _3 = (Parsing.peek_val __caml_parser_env 0 : 'fof_unitary_formula) in
    Obj.repr(
# 324 "parser-hotptp/htparser.mly"
                                                  ( Appl(Appl(Symbol(Signature.conjunction),_1),_3) )
# 1107 "parser-hotptp/htparser.ml"
               : 'fof_and_formula))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 2 : 'fof_or_formula) in
    let _3 = (Parsing.peek_val __caml_parser_env 0 : 'fof_unitary_formula) in
    Obj.repr(
# 328 "parser-hotptp/htparser.mly"
                                             ( Appl(Appl(Symbol(Signature.disjunction),_1),_3) )
# 1115 "parser-hotptp/htparser.ml"
               : 'fof_or_formula))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'fof_unitary_formula) in
    Obj.repr(
# 329 "parser-hotptp/htparser.mly"
                        ( _1 )
# 1122 "parser-hotptp/htparser.ml"
               : 'fof_or_formula))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'fof_quantified_formula) in
    Obj.repr(
# 333 "parser-hotptp/htparser.mly"
                           ( _1 )
# 1129 "parser-hotptp/htparser.ml"
               : 'fof_unitary_formula))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'fof_unary_formula) in
    Obj.repr(
# 334 "parser-hotptp/htparser.mly"
                      ( _1 )
# 1136 "parser-hotptp/htparser.ml"
               : 'fof_unitary_formula))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'fof_atomic_formula) in
    Obj.repr(
# 335 "parser-hotptp/htparser.mly"
                       ( _1 )
# 1143 "parser-hotptp/htparser.ml"
               : 'fof_unitary_formula))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 1 : 'fof_formula) in
    Obj.repr(
# 336 "parser-hotptp/htparser.mly"
                              ( _2 )
# 1150 "parser-hotptp/htparser.ml"
               : 'fof_unitary_formula))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 5 : 'quantifier) in
    let _3 = (Parsing.peek_val __caml_parser_env 3 : 'fof_variable_decls) in
    let _6 = (Parsing.peek_val __caml_parser_env 0 : 'fof_unitary_formula) in
    Obj.repr(
# 340 "parser-hotptp/htparser.mly"
                                                                        ( multi_quantified _1 _3 _6 )
# 1159 "parser-hotptp/htparser.ml"
               : 'fof_quantified_formula))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'fof_variable_decl) in
    Obj.repr(
# 344 "parser-hotptp/htparser.mly"
                      ( [_1] )
# 1166 "parser-hotptp/htparser.ml"
               : 'fof_variable_decls))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 2 : 'fof_variable_decl) in
    let _3 = (Parsing.peek_val __caml_parser_env 0 : 'fof_variable_decls) in
    Obj.repr(
# 345 "parser-hotptp/htparser.mly"
                                               ( _1::_3 )
# 1174 "parser-hotptp/htparser.ml"
               : 'fof_variable_decls))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'fof_variable) in
    Obj.repr(
# 349 "parser-hotptp/htparser.mly"
                 ( (_1,bt_i) )
# 1181 "parser-hotptp/htparser.ml"
               : 'fof_variable_decl))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 1 : 'fof_unary_connective) in
    let _2 = (Parsing.peek_val __caml_parser_env 0 : 'fof_unitary_formula) in
    Obj.repr(
# 353 "parser-hotptp/htparser.mly"
                                             ( Appl(_1,_2) )
# 1189 "parser-hotptp/htparser.ml"
               : 'fof_unary_formula))
; (fun __caml_parser_env ->
    Obj.repr(
# 357 "parser-hotptp/htparser.mly"
          ( Symbol(Signature.neg) )
# 1195 "parser-hotptp/htparser.ml"
               : 'fof_unary_connective))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'fof_prop_const) in
    Obj.repr(
# 361 "parser-hotptp/htparser.mly"
                   ( Symbol _1 )
# 1202 "parser-hotptp/htparser.ml"
               : 'fof_atomic_formula))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 2 : 'fof_term) in
    let _2 = (Parsing.peek_val __caml_parser_env 1 : 'fof_infix_pred) in
    let _3 = (Parsing.peek_val __caml_parser_env 0 : 'fof_term) in
    Obj.repr(
# 362 "parser-hotptp/htparser.mly"
                                     ( Appl(Appl(_2,_1),_3) )
# 1211 "parser-hotptp/htparser.ml"
               : 'fof_atomic_formula))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 3 : 'fof_pred) in
    let _3 = (Parsing.peek_val __caml_parser_env 1 : 'fof_args) in
    Obj.repr(
# 366 "parser-hotptp/htparser.mly"
                                    ( (* print_string ("fof_pred "^(Term.to_hotptp $1)^" with args ");
                                      List.map (fun a -> print_string ((Term.to_hotptp a)^", ")) $3;
                                      print_string "\n"; *)
      add_const (get_symbol _1, mk_funtype (List.map (fun _ -> bt_i) _3) bt_o);
      List.fold_left (fun acc arg -> Appl(acc,arg)) _1 _3 )
# 1223 "parser-hotptp/htparser.ml"
               : 'fof_atomic_formula))
; (fun __caml_parser_env ->
    Obj.repr(
# 374 "parser-hotptp/htparser.mly"
             ( Signature.ctrue )
# 1229 "parser-hotptp/htparser.ml"
               : 'fof_prop_const))
; (fun __caml_parser_env ->
    Obj.repr(
# 375 "parser-hotptp/htparser.mly"
              ( Signature.cfalse )
# 1235 "parser-hotptp/htparser.ml"
               : 'fof_prop_const))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'atomic_word) in
    Obj.repr(
# 376 "parser-hotptp/htparser.mly"
                ( add_const (_1,bt_o); _1 )
# 1242 "parser-hotptp/htparser.ml"
               : 'fof_prop_const))
; (fun __caml_parser_env ->
    Obj.repr(
# 380 "parser-hotptp/htparser.mly"
           ( Symbol(Signature.equality) )
# 1248 "parser-hotptp/htparser.ml"
               : 'fof_infix_pred))
; (fun __caml_parser_env ->
    Obj.repr(
# 381 "parser-hotptp/htparser.mly"
            ( Symbol(Signature.nequals) )
# 1254 "parser-hotptp/htparser.ml"
               : 'fof_infix_pred))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'fof_term) in
    Obj.repr(
# 385 "parser-hotptp/htparser.mly"
             ( [_1] )
# 1261 "parser-hotptp/htparser.ml"
               : 'fof_args))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 2 : 'fof_term) in
    let _3 = (Parsing.peek_val __caml_parser_env 0 : 'fof_args) in
    Obj.repr(
# 386 "parser-hotptp/htparser.mly"
                            ( _1::_3 )
# 1269 "parser-hotptp/htparser.ml"
               : 'fof_args))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'fof_constant) in
    Obj.repr(
# 390 "parser-hotptp/htparser.mly"
                 ( _1 )
# 1276 "parser-hotptp/htparser.ml"
               : 'fof_term))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'fof_variable) in
    Obj.repr(
# 391 "parser-hotptp/htparser.mly"
                 ( Symbol _1 )
# 1283 "parser-hotptp/htparser.ml"
               : 'fof_term))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 3 : 'fof_pred) in
    let _3 = (Parsing.peek_val __caml_parser_env 1 : 'fof_args) in
    Obj.repr(
# 393 "parser-hotptp/htparser.mly"
                                    (
      add_const (get_symbol _1, mk_funtype (List.map (fun _ -> bt_i) _3) bt_i);
      List.fold_left (fun acc arg -> Appl(acc,arg)) _1 _3 )
# 1293 "parser-hotptp/htparser.ml"
               : 'fof_term))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'atomic_word) in
    Obj.repr(
# 399 "parser-hotptp/htparser.mly"
                ( (* print_string ("There's a constant of type $i: "^$1^"\n"); *) add_const (_1, bt_i);
      Symbol _1 )
# 1301 "parser-hotptp/htparser.ml"
               : 'fof_constant))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'number) in
    Obj.repr(
# 401 "parser-hotptp/htparser.mly"
           ( Symbol _1 )
# 1308 "parser-hotptp/htparser.ml"
               : 'fof_constant))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'variable) in
    Obj.repr(
# 405 "parser-hotptp/htparser.mly"
             ( _1 )
# 1315 "parser-hotptp/htparser.ml"
               : 'fof_variable))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'atomic_word) in
    Obj.repr(
# 415 "parser-hotptp/htparser.mly"
                ( Symbol _1 )
# 1322 "parser-hotptp/htparser.ml"
               : 'fof_pred))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'thf_binary_formula) in
    Obj.repr(
# 420 "parser-hotptp/htparser.mly"
                       ( _1 )
# 1329 "parser-hotptp/htparser.ml"
               : 'thf_logic_formula))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'thf_unitary_formula) in
    Obj.repr(
# 421 "parser-hotptp/htparser.mly"
                        ( _1 )
# 1336 "parser-hotptp/htparser.ml"
               : 'thf_logic_formula))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'thf_binary_pair) in
    Obj.repr(
# 426 "parser-hotptp/htparser.mly"
                    ( _1 )
# 1343 "parser-hotptp/htparser.ml"
               : 'thf_binary_formula))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'thf_binary_tuple) in
    Obj.repr(
# 427 "parser-hotptp/htparser.mly"
                     ( _1 )
# 1350 "parser-hotptp/htparser.ml"
               : 'thf_binary_formula))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 2 : 'thf_unitary_formula) in
    let _2 = (Parsing.peek_val __caml_parser_env 1 : 'thf_pair_connective) in
    let _3 = (Parsing.peek_val __caml_parser_env 0 : 'thf_unitary_formula) in
    Obj.repr(
# 431 "parser-hotptp/htparser.mly"
                                                                ( Appl(Appl(_2,_1),_3) )
# 1359 "parser-hotptp/htparser.ml"
               : 'thf_binary_pair))
; (fun __caml_parser_env ->
    Obj.repr(
# 435 "parser-hotptp/htparser.mly"
        ( Symbol(Signature.iff) )
# 1365 "parser-hotptp/htparser.ml"
               : 'thf_pair_connective))
; (fun __caml_parser_env ->
    Obj.repr(
# 436 "parser-hotptp/htparser.mly"
         ( Symbol(Signature.negated_iff) )
# 1371 "parser-hotptp/htparser.ml"
               : 'thf_pair_connective))
; (fun __caml_parser_env ->
    Obj.repr(
# 437 "parser-hotptp/htparser.mly"
            ( Symbol(Signature.implies) )
# 1377 "parser-hotptp/htparser.ml"
               : 'thf_pair_connective))
; (fun __caml_parser_env ->
    Obj.repr(
# 438 "parser-hotptp/htparser.mly"
       ( Symbol(Signature.i_f) )
# 1383 "parser-hotptp/htparser.ml"
               : 'thf_pair_connective))
; (fun __caml_parser_env ->
    Obj.repr(
# 439 "parser-hotptp/htparser.mly"
           ( Symbol(Signature.negated_disjunction) )
# 1389 "parser-hotptp/htparser.ml"
               : 'thf_pair_connective))
; (fun __caml_parser_env ->
    Obj.repr(
# 440 "parser-hotptp/htparser.mly"
               ( Symbol(Signature.negated_conjunction) )
# 1395 "parser-hotptp/htparser.ml"
               : 'thf_pair_connective))
; (fun __caml_parser_env ->
    Obj.repr(
# 441 "parser-hotptp/htparser.mly"
           ( Symbol(Signature.equality) )
# 1401 "parser-hotptp/htparser.ml"
               : 'thf_pair_connective))
; (fun __caml_parser_env ->
    Obj.repr(
# 442 "parser-hotptp/htparser.mly"
            ( Symbol(Signature.nequals) )
# 1407 "parser-hotptp/htparser.ml"
               : 'thf_pair_connective))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'thf_or_formula) in
    Obj.repr(
# 447 "parser-hotptp/htparser.mly"
                   ( _1 )
# 1414 "parser-hotptp/htparser.ml"
               : 'thf_binary_tuple))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'thf_and_formula) in
    Obj.repr(
# 448 "parser-hotptp/htparser.mly"
                    ( _1 )
# 1421 "parser-hotptp/htparser.ml"
               : 'thf_binary_tuple))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'thf_apply_formula) in
    Obj.repr(
# 449 "parser-hotptp/htparser.mly"
                      ( _1 )
# 1428 "parser-hotptp/htparser.ml"
               : 'thf_binary_tuple))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 2 : 'thf_unitary_formula) in
    let _3 = (Parsing.peek_val __caml_parser_env 0 : 'thf_unitary_formula) in
    Obj.repr(
# 453 "parser-hotptp/htparser.mly"
                                                  ( Appl(Appl(Symbol(Signature.disjunction),_1),_3) )
# 1436 "parser-hotptp/htparser.ml"
               : 'thf_or_formula))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 2 : 'thf_or_formula) in
    let _3 = (Parsing.peek_val __caml_parser_env 0 : 'thf_unitary_formula) in
    Obj.repr(
# 454 "parser-hotptp/htparser.mly"
                                             ( Appl(Appl(Symbol(Signature.disjunction),_1),_3) )
# 1444 "parser-hotptp/htparser.ml"
               : 'thf_or_formula))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 2 : 'thf_unitary_formula) in
    let _3 = (Parsing.peek_val __caml_parser_env 0 : 'thf_unitary_formula) in
    Obj.repr(
# 458 "parser-hotptp/htparser.mly"
                                                      ( Appl(Appl(Symbol(Signature.conjunction),_1),_3) )
# 1452 "parser-hotptp/htparser.ml"
               : 'thf_and_formula))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 2 : 'thf_and_formula) in
    let _3 = (Parsing.peek_val __caml_parser_env 0 : 'thf_unitary_formula) in
    Obj.repr(
# 459 "parser-hotptp/htparser.mly"
                                                  ( Appl(Appl(Symbol(Signature.conjunction),_1),_3) )
# 1460 "parser-hotptp/htparser.ml"
               : 'thf_and_formula))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 2 : 'thf_unitary_formula) in
    let _3 = (Parsing.peek_val __caml_parser_env 0 : 'thf_unitary_formula) in
    Obj.repr(
# 463 "parser-hotptp/htparser.mly"
                                                    ( Appl(_1,_3) )
# 1468 "parser-hotptp/htparser.ml"
               : 'thf_apply_formula))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 2 : 'thf_apply_formula) in
    let _3 = (Parsing.peek_val __caml_parser_env 0 : 'thf_unitary_formula) in
    Obj.repr(
# 464 "parser-hotptp/htparser.mly"
                                                  ( Appl(_1,_3) )
# 1476 "parser-hotptp/htparser.ml"
               : 'thf_apply_formula))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'thf_quantified_formula) in
    Obj.repr(
# 468 "parser-hotptp/htparser.mly"
                           ( _1 )
# 1483 "parser-hotptp/htparser.ml"
               : 'thf_unitary_formula))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'thf_abstraction) in
    Obj.repr(
# 469 "parser-hotptp/htparser.mly"
                    ( _1 )
# 1490 "parser-hotptp/htparser.ml"
               : 'thf_unitary_formula))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'thf_unary_formula) in
    Obj.repr(
# 470 "parser-hotptp/htparser.mly"
                      ( _1 )
# 1497 "parser-hotptp/htparser.ml"
               : 'thf_unitary_formula))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'thf_atom) in
    Obj.repr(
# 471 "parser-hotptp/htparser.mly"
             ( _1 )
# 1504 "parser-hotptp/htparser.ml"
               : 'thf_unitary_formula))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 1 : 'thf_logic_formula) in
    Obj.repr(
# 474 "parser-hotptp/htparser.mly"
                                    ( _2 )
# 1511 "parser-hotptp/htparser.ml"
               : 'thf_unitary_formula))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 5 : 'quantifier) in
    let _3 = (Parsing.peek_val __caml_parser_env 3 : 'variable_decls) in
    let _6 = (Parsing.peek_val __caml_parser_env 0 : 'thf_unitary_formula) in
    Obj.repr(
# 478 "parser-hotptp/htparser.mly"
                                                                    ( multi_quantified _1 _3 _6 )
# 1520 "parser-hotptp/htparser.ml"
               : 'thf_quantified_formula))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 5 : 'lambda_symbol) in
    let _3 = (Parsing.peek_val __caml_parser_env 3 : 'variable_decls) in
    let _6 = (Parsing.peek_val __caml_parser_env 0 : 'thf_unitary_formula) in
    Obj.repr(
# 482 "parser-hotptp/htparser.mly"
                                                                       ( multi_abstr _3 _6 )
# 1529 "parser-hotptp/htparser.ml"
               : 'thf_abstraction))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 5 : 'lambda_symbol) in
    let _3 = (Parsing.peek_val __caml_parser_env 3 : 'type_variable_decls) in
    let _6 = (Parsing.peek_val __caml_parser_env 0 : 'thf_unitary_formula) in
    Obj.repr(
# 483 "parser-hotptp/htparser.mly"
                                                                            ( _6 )
# 1538 "parser-hotptp/htparser.ml"
               : 'thf_abstraction))
; (fun __caml_parser_env ->
    let _3 = (Parsing.peek_val __caml_parser_env 1 : 'thf_logic_formula) in
    Obj.repr(
# 487 "parser-hotptp/htparser.mly"
                                          ( Appl(Symbol(Signature.neg),_3) )
# 1545 "parser-hotptp/htparser.ml"
               : 'thf_unary_formula))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'variable) in
    Obj.repr(
# 492 "parser-hotptp/htparser.mly"
             ( Symbol _1 )
# 1552 "parser-hotptp/htparser.ml"
               : 'thf_atom))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'constant) in
    Obj.repr(
# 493 "parser-hotptp/htparser.mly"
             ( Symbol _1 )
# 1559 "parser-hotptp/htparser.ml"
               : 'thf_atom))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'number) in
    Obj.repr(
# 494 "parser-hotptp/htparser.mly"
           ( Symbol _1 )
# 1566 "parser-hotptp/htparser.ml"
               : 'thf_atom))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'binary_connective) in
    Obj.repr(
# 495 "parser-hotptp/htparser.mly"
                      ( _1 )
# 1573 "parser-hotptp/htparser.ml"
               : 'thf_atom))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'unary_connective) in
    Obj.repr(
# 496 "parser-hotptp/htparser.mly"
                     ( _1 )
# 1580 "parser-hotptp/htparser.ml"
               : 'thf_atom))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'lambda_term) in
    Obj.repr(
# 505 "parser-hotptp/htparser.mly"
                ( _1 )
# 1587 "parser-hotptp/htparser.ml"
               : 'term))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'apply_term) in
    Obj.repr(
# 506 "parser-hotptp/htparser.mly"
               ( _1 )
# 1594 "parser-hotptp/htparser.ml"
               : 'term))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 1 : 'term) in
    Obj.repr(
# 507 "parser-hotptp/htparser.mly"
                       ( _2 )
# 1601 "parser-hotptp/htparser.ml"
               : 'term))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'variable) in
    Obj.repr(
# 508 "parser-hotptp/htparser.mly"
             ( Symbol _1 )
# 1608 "parser-hotptp/htparser.ml"
               : 'term))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'constant) in
    Obj.repr(
# 509 "parser-hotptp/htparser.mly"
             ( Symbol _1 )
# 1615 "parser-hotptp/htparser.ml"
               : 'term))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'quantified_term) in
    Obj.repr(
# 510 "parser-hotptp/htparser.mly"
                    ( _1 )
# 1622 "parser-hotptp/htparser.ml"
               : 'term))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 1 : 'unary_connective) in
    let _2 = (Parsing.peek_val __caml_parser_env 0 : 'term) in
    Obj.repr(
# 511 "parser-hotptp/htparser.mly"
                          ( Appl(_1,_2) )
# 1630 "parser-hotptp/htparser.ml"
               : 'term))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 2 : 'term) in
    let _2 = (Parsing.peek_val __caml_parser_env 1 : 'binary_connective) in
    let _3 = (Parsing.peek_val __caml_parser_env 0 : 'term) in
    Obj.repr(
# 512 "parser-hotptp/htparser.mly"
                                ( Appl(Appl(_2,_1),_3) )
# 1639 "parser-hotptp/htparser.ml"
               : 'term))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'number) in
    Obj.repr(
# 513 "parser-hotptp/htparser.mly"
           ( Symbol _1 )
# 1646 "parser-hotptp/htparser.ml"
               : 'term))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'binary_connective) in
    Obj.repr(
# 514 "parser-hotptp/htparser.mly"
                      ( _1 )
# 1653 "parser-hotptp/htparser.ml"
               : 'term))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'unary_connective) in
    Obj.repr(
# 515 "parser-hotptp/htparser.mly"
                     ( _1 )
# 1660 "parser-hotptp/htparser.ml"
               : 'term))
; (fun __caml_parser_env ->
    Obj.repr(
# 520 "parser-hotptp/htparser.mly"
          ( Symbol(Signature.neg) )
# 1666 "parser-hotptp/htparser.ml"
               : 'unary_connective))
; (fun __caml_parser_env ->
    Obj.repr(
# 525 "parser-hotptp/htparser.mly"
        ( Symbol(Signature.iff) )
# 1672 "parser-hotptp/htparser.ml"
               : 'binary_connective))
; (fun __caml_parser_env ->
    Obj.repr(
# 526 "parser-hotptp/htparser.mly"
         ( Symbol(Signature.negated_iff) )
# 1678 "parser-hotptp/htparser.ml"
               : 'binary_connective))
; (fun __caml_parser_env ->
    Obj.repr(
# 527 "parser-hotptp/htparser.mly"
            ( Symbol(Signature.implies) )
# 1684 "parser-hotptp/htparser.ml"
               : 'binary_connective))
; (fun __caml_parser_env ->
    Obj.repr(
# 528 "parser-hotptp/htparser.mly"
       ( Symbol(Signature.i_f) )
# 1690 "parser-hotptp/htparser.ml"
               : 'binary_connective))
; (fun __caml_parser_env ->
    Obj.repr(
# 529 "parser-hotptp/htparser.mly"
          ( Symbol(Signature.disjunction) )
# 1696 "parser-hotptp/htparser.ml"
               : 'binary_connective))
; (fun __caml_parser_env ->
    Obj.repr(
# 530 "parser-hotptp/htparser.mly"
           ( Symbol(Signature.negated_disjunction) )
# 1702 "parser-hotptp/htparser.ml"
               : 'binary_connective))
; (fun __caml_parser_env ->
    Obj.repr(
# 531 "parser-hotptp/htparser.mly"
              ( Symbol(Signature.conjunction) )
# 1708 "parser-hotptp/htparser.ml"
               : 'binary_connective))
; (fun __caml_parser_env ->
    Obj.repr(
# 532 "parser-hotptp/htparser.mly"
               ( Symbol(Signature.negated_conjunction) )
# 1714 "parser-hotptp/htparser.ml"
               : 'binary_connective))
; (fun __caml_parser_env ->
    Obj.repr(
# 533 "parser-hotptp/htparser.mly"
           ( Symbol(Signature.equality) )
# 1720 "parser-hotptp/htparser.ml"
               : 'binary_connective))
; (fun __caml_parser_env ->
    Obj.repr(
# 534 "parser-hotptp/htparser.mly"
            ( Symbol(Signature.nequals) )
# 1726 "parser-hotptp/htparser.ml"
               : 'binary_connective))
; (fun __caml_parser_env ->
    Obj.repr(
# 538 "parser-hotptp/htparser.mly"
        ( Symbol(Signature.iff) )
# 1732 "parser-hotptp/htparser.ml"
               : 'fof_binary_connective))
; (fun __caml_parser_env ->
    Obj.repr(
# 539 "parser-hotptp/htparser.mly"
         ( Symbol(Signature.negated_iff) )
# 1738 "parser-hotptp/htparser.ml"
               : 'fof_binary_connective))
; (fun __caml_parser_env ->
    Obj.repr(
# 540 "parser-hotptp/htparser.mly"
            ( Symbol(Signature.implies) )
# 1744 "parser-hotptp/htparser.ml"
               : 'fof_binary_connective))
; (fun __caml_parser_env ->
    Obj.repr(
# 541 "parser-hotptp/htparser.mly"
       ( Symbol(Signature.i_f) )
# 1750 "parser-hotptp/htparser.ml"
               : 'fof_binary_connective))
; (fun __caml_parser_env ->
    Obj.repr(
# 542 "parser-hotptp/htparser.mly"
           ( Symbol(Signature.negated_disjunction) )
# 1756 "parser-hotptp/htparser.ml"
               : 'fof_binary_connective))
; (fun __caml_parser_env ->
    Obj.repr(
# 543 "parser-hotptp/htparser.mly"
               ( Symbol(Signature.negated_conjunction) )
# 1762 "parser-hotptp/htparser.ml"
               : 'fof_binary_connective))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 5 : 'quantifier) in
    let _3 = (Parsing.peek_val __caml_parser_env 3 : 'variable_decls) in
    let _6 = (Parsing.peek_val __caml_parser_env 0 : 'term) in
    Obj.repr(
# 547 "parser-hotptp/htparser.mly"
                                                     ( multi_quantified _1 _3 _6 )
# 1771 "parser-hotptp/htparser.ml"
               : 'quantified_term))
; (fun __caml_parser_env ->
    Obj.repr(
# 551 "parser-hotptp/htparser.mly"
                ( Symbol(Signature.forall) )
# 1777 "parser-hotptp/htparser.ml"
               : 'quantifier))
; (fun __caml_parser_env ->
    Obj.repr(
# 552 "parser-hotptp/htparser.mly"
             ( Symbol(Signature.exists) )
# 1783 "parser-hotptp/htparser.ml"
               : 'quantifier))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 2 : 'term) in
    let _3 = (Parsing.peek_val __caml_parser_env 0 : 'term) in
    Obj.repr(
# 556 "parser-hotptp/htparser.mly"
                      ( Appl(_1,_3) )
# 1791 "parser-hotptp/htparser.ml"
               : 'apply_term))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 5 : 'lambda_symbol) in
    let _3 = (Parsing.peek_val __caml_parser_env 3 : 'variable_decls) in
    let _6 = (Parsing.peek_val __caml_parser_env 0 : 'term) in
    Obj.repr(
# 560 "parser-hotptp/htparser.mly"
                                                        ( multi_abstr _3 _6 )
# 1800 "parser-hotptp/htparser.ml"
               : 'lambda_term))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 5 : 'lambda_symbol) in
    let _3 = (Parsing.peek_val __caml_parser_env 3 : 'type_variable_decls) in
    let _6 = (Parsing.peek_val __caml_parser_env 0 : 'term) in
    Obj.repr(
# 561 "parser-hotptp/htparser.mly"
                                                             ( _6 )
# 1809 "parser-hotptp/htparser.ml"
               : 'lambda_term))
; (fun __caml_parser_env ->
    Obj.repr(
# 565 "parser-hotptp/htparser.mly"
           ( )
# 1815 "parser-hotptp/htparser.ml"
               : 'lambda_symbol))
; (fun __caml_parser_env ->
    Obj.repr(
# 566 "parser-hotptp/htparser.mly"
          ( )
# 1821 "parser-hotptp/htparser.ml"
               : 'lambda_symbol))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'variable_decl) in
    Obj.repr(
# 571 "parser-hotptp/htparser.mly"
                  ( [_1] )
# 1828 "parser-hotptp/htparser.ml"
               : 'variable_decls))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 2 : 'variable_decl) in
    let _3 = (Parsing.peek_val __caml_parser_env 0 : 'variable_decls) in
    Obj.repr(
# 572 "parser-hotptp/htparser.mly"
                                       ( _1::_3 )
# 1836 "parser-hotptp/htparser.ml"
               : 'variable_decls))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 2 : 'variable) in
    let _3 = (Parsing.peek_val __caml_parser_env 0 : 'type_expr) in
    Obj.repr(
# 576 "parser-hotptp/htparser.mly"
                             ( (_1,_3) )
# 1844 "parser-hotptp/htparser.ml"
               : 'variable_decl))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'type_variable_decl) in
    Obj.repr(
# 580 "parser-hotptp/htparser.mly"
                       ( )
# 1851 "parser-hotptp/htparser.ml"
               : 'type_variable_decls))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 2 : 'type_variable_decl) in
    let _3 = (Parsing.peek_val __caml_parser_env 0 : 'type_variable_decls) in
    Obj.repr(
# 581 "parser-hotptp/htparser.mly"
                                                 ( )
# 1859 "parser-hotptp/htparser.ml"
               : 'type_variable_decls))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 2 : 'variable) in
    Obj.repr(
# 585 "parser-hotptp/htparser.mly"
                            ( add_typevar _1 )
# 1866 "parser-hotptp/htparser.ml"
               : 'type_variable_decl))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'atomic_type) in
    Obj.repr(
# 589 "parser-hotptp/htparser.mly"
                ( _1 )
# 1873 "parser-hotptp/htparser.ml"
               : 'type_expr))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'type_variable) in
    Obj.repr(
# 590 "parser-hotptp/htparser.mly"
                  ( if Hashtbl.mem !bound_typevars _1 then mk_polyvar (Hashtbl.find !bound_typevars _1) else basetype _1  )
# 1880 "parser-hotptp/htparser.ml"
               : 'type_expr))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'poly_type_variable) in
    Obj.repr(
# 591 "parser-hotptp/htparser.mly"
                       ( basetype _1 )
# 1887 "parser-hotptp/htparser.ml"
               : 'type_expr))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 2 : 'type_expr) in
    let _2 = (Parsing.peek_val __caml_parser_env 1 : 'map_arrow) in
    let _3 = (Parsing.peek_val __caml_parser_env 0 : 'type_expr) in
    Obj.repr(
# 592 "parser-hotptp/htparser.mly"
                                  ( abstr_type _1 _3 )
# 1896 "parser-hotptp/htparser.ml"
               : 'type_expr))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 1 : 'type_expr) in
    Obj.repr(
# 593 "parser-hotptp/htparser.mly"
                            ( _2 )
# 1903 "parser-hotptp/htparser.ml"
               : 'type_expr))
; (fun __caml_parser_env ->
    Obj.repr(
# 597 "parser-hotptp/htparser.mly"
          ( bt_o )
# 1909 "parser-hotptp/htparser.ml"
               : 'atomic_type))
; (fun __caml_parser_env ->
    Obj.repr(
# 598 "parser-hotptp/htparser.mly"
          ( bt_i )
# 1915 "parser-hotptp/htparser.ml"
               : 'atomic_type))
; (fun __caml_parser_env ->
    Obj.repr(
# 599 "parser-hotptp/htparser.mly"
            ( basetype "$int" )
# 1921 "parser-hotptp/htparser.ml"
               : 'atomic_type))
; (fun __caml_parser_env ->
    Obj.repr(
# 600 "parser-hotptp/htparser.mly"
             ( basetype "$real" )
# 1927 "parser-hotptp/htparser.ml"
               : 'atomic_type))
; (fun __caml_parser_env ->
    Obj.repr(
# 601 "parser-hotptp/htparser.mly"
             ( bt_type )
# 1933 "parser-hotptp/htparser.ml"
               : 'atomic_type))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : string) in
    Obj.repr(
# 602 "parser-hotptp/htparser.mly"
               ( basetype _1 )
# 1940 "parser-hotptp/htparser.ml"
               : 'atomic_type))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : string) in
    Obj.repr(
# 606 "parser-hotptp/htparser.mly"
              ( _1 )
# 1947 "parser-hotptp/htparser.ml"
               : 'type_variable))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : string) in
    Obj.repr(
# 610 "parser-hotptp/htparser.mly"
               ( _1 )
# 1954 "parser-hotptp/htparser.ml"
               : 'poly_type_variable))
; (fun __caml_parser_env ->
    Obj.repr(
# 615 "parser-hotptp/htparser.mly"
           ( )
# 1960 "parser-hotptp/htparser.ml"
               : 'map_arrow))
; (fun __caml_parser_env ->
    Obj.repr(
# 616 "parser-hotptp/htparser.mly"
             ( )
# 1966 "parser-hotptp/htparser.ml"
               : 'map_arrow))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'atomic_word) in
    Obj.repr(
# 620 "parser-hotptp/htparser.mly"
                ( _1 )
# 1973 "parser-hotptp/htparser.ml"
               : 'name))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : string) in
    Obj.repr(
# 621 "parser-hotptp/htparser.mly"
                     ( _1 )
# 1980 "parser-hotptp/htparser.ml"
               : 'name))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : string) in
    Obj.repr(
# 625 "parser-hotptp/htparser.mly"
               ( _1 )
# 1987 "parser-hotptp/htparser.ml"
               : 'atomic_word))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : string) in
    Obj.repr(
# 626 "parser-hotptp/htparser.mly"
                  ( _1 )
# 1994 "parser-hotptp/htparser.ml"
               : 'atomic_word))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : string) in
    Obj.repr(
# 630 "parser-hotptp/htparser.mly"
               ( _1 )
# 2001 "parser-hotptp/htparser.ml"
               : 'variable))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : string) in
    Obj.repr(
# 634 "parser-hotptp/htparser.mly"
                     ( add_const (_1, basetype "$int");  _1 )
# 2008 "parser-hotptp/htparser.ml"
               : 'number))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : string) in
    Obj.repr(
# 635 "parser-hotptp/htparser.mly"
                   ( add_const (_1, basetype "$int"); _1 )
# 2015 "parser-hotptp/htparser.ml"
               : 'number))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : string) in
    Obj.repr(
# 636 "parser-hotptp/htparser.mly"
         ( add_const (_1, basetype "$real"); _1 )
# 2022 "parser-hotptp/htparser.ml"
               : 'number))
; (fun __caml_parser_env ->
    Obj.repr(
# 638 "parser-hotptp/htparser.mly"
       ()
# 2028 "parser-hotptp/htparser.ml"
               : 'null))
(* Entry input *)
; (fun __caml_parser_env -> raise (Parsing.YYexit (Parsing.peek_val __caml_parser_env 0)))
|]
let yytables =
  { Parsing.actions=yyact;
    Parsing.transl_const=yytransl_const;
    Parsing.transl_block=yytransl_block;
    Parsing.lhs=yylhs;
    Parsing.len=yylen;
    Parsing.defred=yydefred;
    Parsing.dgoto=yydgoto;
    Parsing.sindex=yysindex;
    Parsing.rindex=yyrindex;
    Parsing.gindex=yygindex;
    Parsing.tablesize=yytablesize;
    Parsing.table=yytable;
    Parsing.check=yycheck;
    Parsing.error_function=parse_error;
    Parsing.names_const=yynames_const;
    Parsing.names_block=yynames_block }
let input (lexfun : Lexing.lexbuf -> token) (lexbuf : Lexing.lexbuf) =
   (Parsing.yyparse yytables 1 lexfun lexbuf : Term.term list * Signature.signature * (string , string * Term.term) Hashtbl.t)
